@@ -273,7 +273,7 @@ void QFcitxInputContext::update()
         }
     }
 
-    QTimer::singleShot(0, this, SLOT(updateCursor()));
+    QMetaObject::invokeMethod(this, "updateCursor", Qt::QueuedConnection);
 }
 
 void QFcitxInputContext::updateCursor()
@@ -500,7 +500,9 @@ bool QFcitxInputContext::x11FilterEvent(QWidget* keywidget, XEvent* event)
                                           event->xkey.time
                                       );
     if (Q_UNLIKELY(m_syncMode)) {
-        result.waitForFinished();
+        do {
+            QCoreApplication::processEvents (QEventLoop::WaitForMoreEvents);
+        } while (QCoreApplication::hasPendingEvents () || !result.isFinished ());
 
         if (!m_connection->isConnected() || !result.isFinished() || result.isError() || result.value() <= 0) {
             return x11FilterEventFallback(event, sym);
@@ -536,7 +538,7 @@ void QFcitxInputContext::x11ProcessKeyEventCallback(QDBusPendingCallWatcher* wat
         delete pkwatcher;
     else {
         pkwatcher->event->xkey.state |= FcitxKeyState_IgnoredMask;
-        QTimer::singleShot(0, pkwatcher, SLOT(processEvent()));
+        QMetaObject::invokeMethod(pkwatcher, "processEvent", Qt::QueuedConnection);
     }
 }
 
