@@ -666,6 +666,9 @@ boolean FcitxInstanceSetCurrentIC(FcitxInstance* instance, FcitxInputContext* ic
     FcitxContextState prevstate = FcitxInstanceGetCurrentState(instance);
     boolean changed = (instance->CurrentIC != ic);
 
+    if (instance->CurrentIC) {
+        FcitxInstanceSetLastIC(instance, instance->CurrentIC);
+    }
     instance->CurrentIC = ic;
 
     FcitxContextState nextstate = FcitxInstanceGetCurrentState(instance);
@@ -680,6 +683,20 @@ boolean FcitxInstanceSetCurrentIC(FcitxInstance* instance, FcitxInputContext* ic
     return changed;
 }
 
+void FcitxInstanceSetLastIC(FcitxInstance* instance, FcitxInputContext* ic)
+{
+    instance->lastIC = ic;
+
+    free(instance->delayedIM);
+    instance->delayedIM = NULL;
+}
+
+void FcitxInstanceSetDelayedIM(FcitxInstance* instance, const char* delayedIM)
+{
+    fcitx_utils_string_swap(&instance->delayedIM, delayedIM);
+}
+
+
 void FcitxInstanceInitBuiltContext(FcitxInstance* instance)
 {
     FcitxInstanceRegisterWatchableContext(instance, CONTEXT_ALTERNATIVE_PREVPAGE_KEY, FCT_Hotkey, FCF_ResetOnInputMethodChange);
@@ -806,6 +823,8 @@ static void FcitxInstanceInitNoPreeditApps(FcitxInstance* instance) {
         re = NULL;
     }
 
+    fcitx_utils_free_string_list(app_pat_list);
+
     instance->no_preedit_app_list = no_preedit_app_list;
 }
 
